TipTacDropDowns = {};

local cfg = TipTac_Config;
local info = {};

-- Shared Media
local SML = LibStub and LibStub("LibSharedMedia-2.0",1);

-- DropDown Lists
TipTacDropDowns.AnchorType = {
	["Normal Anchor"] = "normal",
	["Mouse Anchor"] = "mouse",
	["Smart Anchor"] = "smart",
};

TipTacDropDowns.AnchorPos = {
	["Top"] = "TOP",
	["Top Left"] = "TOPLEFT",
	["Top Right"] = "TOPRIGHT",
	["Bottom"] = "BOTTOM",
	["Bottom Left"] = "BOTTOMLEFT",
	["Bottom Right"] = "BOTTOMRIGHT",
	["Left"] = "LEFT",
	["Right"] = "RIGHT",
	["Center"] = "CENTER",
};

--------------------------------------------------------------------------------------------------------
--                                            Default Init                                            --
--------------------------------------------------------------------------------------------------------

local function Default_SelectValue(arg1,arg2)
	UIDropDownMenu_SetSelectedValue(arg1,this.value);
	cfg[arg1.option.var] = this.value;
	TipTac_ApplySettings();
end

function TipTacDropDowns:Default()
	local dropDown = TipTacOptions.lastDropDown or this:GetParent();
	info.func = Default_SelectValue;
	info.arg1 = dropDown;
	for text, option in pairs(dropDown.option.list) do
		info.text = text;
		info.value = option;
		info.checked = (cfg[dropDown.option.var] == option);
		UIDropDownMenu_AddButton(info);
	end
end

--------------------------------------------------------------------------------------------------------
--                                           Font Face Init                                           --
--------------------------------------------------------------------------------------------------------

function TipTacDropDowns:FontFace()
	local dropDown = TipTacOptions.lastDropDown or this:GetParent();
	info.func = Default_SelectValue;
	info.arg1 = dropDown;

	if (SML) then
		for _, name in pairs(SML:List("font")) do
			info.text = name;
			info.value = SML:Fetch("font",name);
			info.checked = (cfg[dropDown.option.var] == name);
			UIDropDownMenu_AddButton(info);
		end
	else
		local fonts = {
			["Friz Quadrata TT"] = "Fonts\\FRIZQT__.TTF",
			["Arial Narrow"] = "Fonts\\ARIALN.TTF",
			["Skurri"] = "Fonts\\SKURRI.TTF",
			["Morpheus"] = "Fonts\\MORPHEUS.TTF",
		};

		for name, font in pairs(fonts) do
			info.text = name;
			info.value = font;
			info.checked = (cfg[dropDown.option.var] == font);
			UIDropDownMenu_AddButton(info);
		end
	end
end

--------------------------------------------------------------------------------------------------------
--                                           Layout Presets                                           --
--------------------------------------------------------------------------------------------------------

local presets = {
	-- TipTac Layout
	["TipTac (Default)"] = {
		fontFace = "Fonts\\FRIZQT__.TTF",
		fontSize = 12,
		fontFlags = "NORMAL",
		showTarget = "second",
		tipTacColor = { 0.1, 0.1, 0.2, 1.0 };
		tipTacBorderColor = { 0.3, 0.3, 0.4, 1.0 };
		tipColor = { 0.1, 0.1, 0.2, 1.0 };
		tipBorderColor = { 0.3, 0.3, 0.4, 1.0 };
		colNormal = "|cff25c1eb";
		colSameGuild = "|cffff32ff";
		colRace = "|cffffffff";
		colLevel = "|cffc0c0c0";
	},
	-- TinyTip Layout
	["TinyTip"] = {
		showTarget = "last",
		tipTacColor = { 0, 0, 0.5, 1.0 };
		tipTacBorderColor = { 0, 0, 0.75, 1.0 };
		tipColor = { 0, 0, 0, 1 };
		tipBorderColor = { 0, 0, 0, 1 };
		colNormal = "|cff00aaff";
		--colSameGuild = "|cffff32ff";
		colRace = "|cffddeeaa";
		colLevel = "|cffffcc00";
		anchorTypeUnit = "mouse",
		anchorPointUnit = "BOTTOM",
	},
	-- TipBuddy Layout
	["TipBuddy"] = {
		fontFace = "Fonts\\FRIZQT__.TTF",
		fontSize = 12,
		fontFlags = "NORMAL",
		showTarget = "second",
		tipTacColor = { 0.1, 0.22, 0.35, 0.78 };
		tipTacBorderColor = { 0.8, 0.8, 0.9, 1.0 };
		tipColor = { 0.1, 0.1, 0.1, 0.8 };
		tipBorderColor = { 0.8, 0.8, 0.9, 1.0 };
		colNormal = "|cff25c1eb";
		colSameGuild = "|cffff32ff";
		colRace = "|cffffffff";
		colLevel = "|cffc0c0c0";
	},
};

local function Layout_SelectValue(arg1,arg2)
	for name, value in pairs(presets[this.value]) do
		cfg[name] = value;
	end
	TipTac_ApplySettings();
	TipTacOptions:BuildCategoryPage();
	UIDropDownMenu_SetSelectedValue(arg1,this.value);
end

function TipTacDropDowns:Layouts()
	info.func = Layout_SelectValue;
	info.arg1 = this:GetParent();
	for name in pairs(presets) do
		info.text = name;
		info.value = name;
		info.checked = nil;
		UIDropDownMenu_AddButton(info);
	end
end